
/*
 * Copyright (c) 2008 - 2012, Andy Bierman, All Rights Reserved.
 * Copyright (c) 2012 - 2021, YumaWorks, Inc., All Rights Reserved.
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

/**
 * @file y_error-example.c
 * @brief YumaPro SIL-SA module for module error-example
 *

*** Generated by yangdump-sdk jaguar-tony-ypw-1872-silsa-errors-2021-11-19.12.48-M-THD

<pre>

    YumaPro SIL-SA module
    module error-example
    revision 2021-11-19
    prefix err
    namespace http://netconfcentral.org/ns/error-example
    Created: 2021-11-20T00:13:32Z
    CLI parameters:
        format yc
        indent 4
        log-level debug3
        module error-example
        output y_error-example.c
        sil-edit2
        sil-get2
        sil-sa
        unified true

  Short Name Mappings
    test_error = /test-silsa-errors/test-error
    test_silsa_errors = /test-silsa-errors
</pre>

 */

#include <xmlstring.h>

#include "procdefs.h"
#include "agt.h"
#include "agt_action.h"
#include "agt_cb.h"
#include "agt_cfg.h"
#include "agt_rpc.h"
#include "agt_sil_lib.h"
#include "agt_timer.h"
#include "agt_util.h"
#include "agt_val.h"
#include "cfg.h"
#include "dlq.h"
#include "getcb.h"
#include "ncx.h"
#include "ncx_feature.h"
#include "ncxmod.h"
#include "ncxtypes.h"
#include "obj.h"
#include "rpc.h"
#include "sil_sa.h"
#include "status.h"
#include "val.h"
#include "val_child.h"
#include "val_util.h"
#include "xml_val.h"
#include "u_error-example.h"
#include "y_error-example.h"


/**
 * @addtogroup yang-library YANG Library
 *
 * Collection of modules supported by the server.
 * Generated by yangdump-sdk.
 */



/**
 * @addtogroup silsa-error-example Module error-example
 * @ingroup yang-library
 *
 * @{
 *
 * YANG instrumentation library.
 * Generated by yangdump-sdk.
 */


/* module static variables */
static ncx_module_t *error_example_mod;
static obj_template_t *test_silsa_errors_obj;
static val_value_t *test_silsa_errors_val;

/**
 * @brief Initialize module static variables
 *
 * Called by server when module is loaded.
 */
static void y_err_init_static_vars (void)
{
    error_example_mod = NULL;
    test_silsa_errors_obj = NULL;
    test_silsa_errors_val = NULL;

} /* y_err_init_static_vars */


/**
 * @brief Edit database object callback (agt_cb_fn_t)\n
 * Path: container /test-silsa-errors
 *
 * @param scb session control block making the request
 * @param msg message in progress for this edit request
 * @param cbtyp callback type for this callback
 * @param editop the parent edit-config operation type,
 * which is also used for all other callbacks
 * that operate on objects.
 * @param newval container object holding the proposed changes
 * to apply to the current config, depending on
 * the editop value. Will not be NULL.
 * @param curval current container values from the "<running>"
 * or "<candidate>" configuration, if any. Could be NULL
 * for create and other operations.
 * @return return status for the phase.
 */
static status_t test_silsa_errors_edit (
    ses_cb_t *scb,
    rpc_msg_t *msg,
    agt_cbtyp_t cbtyp,
    op_editop_t editop,
    val_value_t *newval,
    val_value_t *curval)
{
    status_t res = NO_ERR;
    val_value_t *errorval = (curval) ? curval : newval;
    uint32 cur_errs = agt_get_error_count(msg);

    if (LOGDEBUG) {
        log_debug("\nEnter test_silsa_errors_edit callback for %s phase",
            agt_cbtype_name(cbtyp));
    }

    res = u_err_test_silsa_errors_edit(scb, msg, cbtyp, editop, newval, curval);

    if (res == NO_ERR && cbtyp == AGT_CB_COMMIT) {
        res = agt_check_cache(&test_silsa_errors_val, newval, curval, editop);
    }


    if ((res != NO_ERR) && (cur_errs == agt_get_error_count(msg))) {
        agt_record_error(
            scb,
            &msg->mhdr,
            NCX_LAYER_CONTENT,
            res,
            NULL,
            (errorval) ? NCX_NT_VAL : NCX_NT_NONE,
            errorval,
            (errorval) ? NCX_NT_VAL : NCX_NT_NONE,
            errorval);
    }
    return res;

} /* test_silsa_errors_edit */


/**
 * @brief Phase 1: Initialize the error-example server instrumentation library.
 *
 * Called by server when module is loaded.
 *
 * @param modname requested module name to load
 * @param revision requested revision date of the module to load.
 * This may be NULL if the module has no revision statements.
 * @return return status. An error will cause the module load to fail.
 */
status_t y_error_example_init (
    const xmlChar *modname,
    const xmlChar *revision)
{
    status_t res = NO_ERR;

    if (LOGDEBUG) {
        log_debug("\nEnter y_error_example_init");
    }


    y_err_init_static_vars();

    /* change if custom handling done */
    if (xml_strcmp(modname, y_err_M_err)) {
        return ERR_NCX_UNKNOWN_MODULE;
    }

    if (revision && xml_strcmp(revision, y_err_R_err)) {
        return ERR_NCX_WRONG_VERSION;
    }

    res = ncxmod_load_module(
        y_err_M_err,
        y_err_R_err,
        agt_get_savedevQ(),
        &error_example_mod);
    if (res != NO_ERR) {
        return res;
    }

    test_silsa_errors_obj = ncx_find_object(
        error_example_mod,
        y_err_N_test_silsa_errors);
    if (test_silsa_errors_obj == NULL) {
        return ERR_NCX_DEF_NOT_FOUND;
    }

    res = agt_cb_register_edit2_callback(
        y_err_M_err,
        (const xmlChar *)"/err:test-silsa-errors",
        y_err_R_err,
        test_silsa_errors_edit);
    if (res != NO_ERR) {
        return res;
    }

    res = u_error_example_init(modname, revision);
    return res;

} /* y_error_example_init */


/**
 * @brief Phase 2: Initialize the error-example server instrumentation library.
 *
 * SIL init phase 2: non-config data structures.
 * Called after running config is loaded.
 *
 * @return return status. An error will cause the
 * server initialization to fail.
 */
status_t y_error_example_init2 (void)
{
    status_t res = NO_ERR;

    if (LOGDEBUG) {
        log_debug("\nEnter y_error_example_init2");
    }


    res = u_error_example_init2();

    return res;

} /* y_error_example_init2 */


/**
 * @brief Cleanup the error-example server instrumentation library.
 *
 * Called by server when module is unloaded.
 *
 */
void y_error_example_cleanup (void)
{

    if (LOGDEBUG) {
        log_debug("\nEnter y_error_example_cleanup");
    }


    agt_cb_unregister_callbacks(
        y_err_M_err,
        (const xmlChar *)"/err:test-silsa-errors");
    u_error_example_cleanup();

} /* y_error_example_cleanup */


 /** @} */

/* END SIL-SA y_error-example.c */
